"""
Authors: Christopher Re and Ben Recht
Copyright 2010-2011 Christopher Re and Ben Recht

This code is released under the GPLv3 License.
http://www.gnu.org/licenses/gpl.html

The files in this directory and its subdirectories are part of Jellyfish.

Jellyifsh is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Jellyfish is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jellyfish.  If not, see <http://www.gnu.org/licenses/>.
"""
def jellyfish(numr,alpha,truerank,beta,noise_var_dec,factor_type):

    import time
    import numpy
    from numpy import random, argsort, zeros
    import scipy.linalg
    from math import pow, sqrt, log, floor, ceil
    import fvector as fw

    t0 = time.time()

    train_base="../scale_data_mpc/data/train"
    probe_base="../scale_data_mpc/data/probe"

    noise_var_print='0.'
    for k in range(noise_var_dec-1):
        noise_var_print += '0'
    noise_var_print+='1'

    noise_var = pow(10.0,-noise_var_dec)

    extension = '.nr='+`numr`+'.alpha='+`alpha`+'.rrank='+`truerank`+\
        '.beta='+`beta`+'.sigma_squared='+noise_var_print+\
        '.factor_type='+`factor_type`+'.data'

    train_file=train_base+extension
    probe_file=probe_base+extension

    t1 = time.time()
    print "loading {0} and {1}".format(train_file,probe_file)
    eh = fw.ExampleHolder(train_file)
    th = fw.TestHandler(probe_file)
    load_time = time.time() - t1
    print "done in {0}s".format(load_time)

    numc = alpha*numr
    M = beta*truerank*(numr +numc-truerank)

    maxepochs = 40
    upper_gap = 10.0
    lower_gap = 6.0
    mrank =  int(floor((numr+numc-sqrt((numr+numc)*(numr+numc)-4*M/3))/2));
    muBound = 1e-5

    tp = fw.tps(example_holder=eh,
                test_handler=th, nEpochs=5, max_rank=mrank, bTraceNorm=True, 
                nSplits=10, nShufflers=2,
                seed=0, initial_stepsize=1e-1, step_diminish=0.9, 
                b=150, mu=muBound)

    m = fw.call_jellyfish(tp)

    row_oversample = int(ceil(2*numr*log(numr)))
    row_order = range(numr)
    random.shuffle(row_order)
    
    s = scipy.linalg.svd(m.L[row_order[0:row_oversample],:], \
                             full_matrices=False, compute_uv=False, 
                         overwrite_a=False)

    diff = []
    beta = 20.0/log(10.0)
    for k in range(mrank-1):
        diff.append( beta*(log(s[k])-log(s[k+1])) )
    
    I = argsort(diff)

    print "SV Gap diff {0:.3f}, {1:.3f}".format(diff[I[-1]],diff[I[-2]])

    if (diff[I[-1]]>upper_gap) & (diff[I[-2]]<lower_gap):
        guessed_rank = I[-1]+1
        print "Setting rank to {0}".format(guessed_rank)
        
        tp = fw.tps(example_holder=eh,
                    test_handler=th, nEpochs=maxepochs, max_rank=guessed_rank, 
                    bTraceNorm=True,
                    nSplits=10, nShufflers=2,
                    seed=0, initial_stepsize=1e-1, step_diminish=0.9,
                    b=150, mu=muBound)
        m = fw.call_jellyfish(tp)
    else:
        print "Keeping rank at {0}".format(mrank)
        
        tp = fw.tps(example_holder=eh,
                    test_handler=th, nEpochs=maxepochs-5, max_rank=mrank, 
                    bTraceNorm=True,
                    nSplits=10, nShufflers=2,
                    seed=0, initial_stepsize=1e-1, step_diminish=0.9,
                    b=150, mu=muBound)
        m = fw.call_jellyfish(tp,initial_guess=m)

    total_time = time.time() - t0
    print "Total Time: {0}s".format(total_time)
    
    return
