"""
Authors: Christopher Re and Ben Recht
Copyright 2010-2011 Christopher Re and Ben Recht

This code is released under the GPLv3 License.
http://www.gnu.org/licenses/gpl.html

The files in this directory and its subdirectories are part of Jellyfish.

Jellyifsh is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Jellyfish is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jellyfish.  If not, see <http://www.gnu.org/licenses/>.
"""
import gen_mpc
from math import *
import os

myemail = "chrisre@cs.wisc.edu"

def mail_msg(msg, subj, receipt):
    return "echo \"{0}\" | mail -s \"{1}\" {2}".format(msg, subj, receipt)

def compute_nSamples(nr, rrank, alpha, oversample):
    return oversample*rrank*((1+alpha)*nr - rrank)

def compute_mu(nr, noise_var, alpha, nSamples):
    #   return 10.5*sqrt(noise_var * nSamples * log(nr)/nr)
    return 1.64*sqrt( noise_var * nSamples * log( nr * (1+alpha) )/nr )*1e-3

def generate_params():
    for nr in [1000,10000,100000]:
        for alpha in [1, 5, 10]:
            for rrank in [5,10,20]:
                for oversample in [3, 5, 10]:
                    for noise_var in [1e-2, 1e-3, 1e-4]:
                        for factor_type in [0,1]:
                            yield (nr,alpha,rrank,oversample,noise_var, factor_type)

def generate_filenames(nr,alpha,rrank,oversample,noise_var,factor_type, data_dir='data', output_dir='output',output_name='output'):
    base_name   = "nr={0}.alpha={1}.rrank={2}.beta={3}.sigma_squared={4}.factor_type={5}".format(nr,alpha,rrank,oversample,noise_var,factor_type)
    train_name  = "{1}/train.{0}.data".format(base_name, data_dir)
    probe_name  = "{1}/probe.{0}.data".format(base_name, data_dir)
    output_name = "{1}/{2}.{0}.txt".format(base_name, output_dir,output_name)
    return (train_name, probe_name,output_name)


def generate_bulk():
    for (nr,alpha, rrank, oversample, noise_var, factor_type) in generate_params():
        (train_name, probe_name,output_name) = generate_filenames(nr,alpha,rrank,oversample,noise_var,factor_type)
        print "Starting {0}".format(train_name)
        gen_mpc.gen_instance(train_name, probe_name, nr, alpha, rrank, factor_type, oversample, noise_var)                           
                           
def generate_jumbo():
    nr = int(1e6); alpha = 1; oversample = 5; rrank = 10; noise_var = 1e-3;
    factor_type = 0
    base_name   = "nr={0}.alpha={1}.rrank={2}.beta={3}.sigma_squared={4}.factor_type={5}".format(nr,alpha,rrank,oversample,noise_var,factor_type)
    train_name  = "data/train.{0}.data".format(base_name)
    probe_name  = "data/probe.{0}.data".format(base_name)
    print "Starting {0}".format(base_name)
    gen_mpc.gen_instance(train_name, probe_name, nr, alpha, rrank, factor_type, oversample, noise_var)
                   

def run_tracenorm(exec_name, nthreads, epochs=100, diminish=0.9):
    for (nr,alpha, rrank, oversample, noise_var, factor_type) in generate_params():
        nSamples = compute_nSamples(nr, rrank, alpha, oversample)
        mu = compute_mu(nr, noise_var, alpha, nSamples)
        (train_name, probe_name, output_name) = generate_filenames(nr,alpha,rrank,oversample,noise_var,factor_type, output_name="tracenorm")
        os.system("{0} --epochs {6} --step_decay {7} --tracenorm 1 --mu {1} --splits {2}  {3} {4} > {5}".format(exec_name, mu, nthreads, train_name, probe_name, output_name, epochs, diminish))

def run_maxnorm(exec_name, nthreads, epochs=100, diminish=0.9):
    for (nr,alpha, rrank, oversample, noise_var, factor_type) in generate_params():
        (train_name, probe_name, output_name) = generate_filenames(nr,alpha,rrank,oversample,noise_var,factor_type, output_name="maxnorm")
        os.system("{0} --epochs {5} --step_decay {6} --splits {1} {2} {3} > {4}".format(exec_name, nthreads, train_name, probe_name, output_name, epochs, diminish))

if __name__ == "__main__":
    generate_bulk()
    # generate_jumbo()
