/* Authors: Christopher Re and Ben Recht
Copyright 2010-2011 Christopher Re and Ben Recht

This code is released under the GPLv3 License.
http://www.gnu.org/licenses/gpl.html

The files in this directory and its subdirectories are part of Jellyfish.

Jellyifsh is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Jellyfish is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jellyfish.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdlib.h>
#include <iostream>
#include <math.h>
#include <stdio.h>
#include <assert.h>
#include "generate_mpc_data.h"

#include "simple_random.h"

simple_random sr;

double dot(int len, double *x, double *y) {
  double ret = 0.0;
  for(int i = len -1; i >= 0; i--) {
    ret += x[i]*y[i];
  }
  return ret;
}

// http://www.taygeta.com/random/gaussian.html
void
gaussian(double &y1, double &y2) {
  double x1, x2, w;
  
  do {
    x1 = 2.0 * sr.rand_double() - 1.0;
    x2 = 2.0 * sr.rand_double() - 1.0;
    w = x1 * x1 + x2 * x2;
  } while ( w >= 1.0 );

  w = sqrt( (-2.0 * log( w ) ) / w );
  y1 = x1 * w;
  y2 = x2 * w;
  
}


void
sample_to_file(char *fname, int nSamples, 
	       int numr, int numc, int rank, 
	       double *L, double *R, double noise_std) {

  
  // std::cerr << "Opening " << fname << std::endl;
  FILE *f = fopen(fname, "w+");
  assert(f != NULL);
  double x,y;
  for(int i = nSamples - 1; i >= 0; i--) {
    int r = rand() % numr;
    int c = rand() % numc;
    gaussian(x,y);
    double v = dot(rank, &L[r*rank], &R[c*rank]) + noise_std*x;
    // NB: the 1 index stuff is for Matlab
    fprintf(f, "%d %d %f\n", r+1, c+1, v);
    
  }
 fclose(f);
}

double sign(double v) {
  return (v > 0.0) ? 1.0 : -1.0;
}

extern "C" 
int
generate_instance(char* train_fname, char* probe_fname, 
		  int numr, double alpha, 
		  int rrank, int factor_type, 
		  double oversample, double noise_var) {
  int numc = (int) ceil(alpha*numr);
  
  double *YL = new double[numr * rrank];
  double *YR = new double[numc * rrank];

  double x=0.0, y=0.0;
  double matrix_scale = pow(rrank, 0.25); // why is this here?
  for(int r = rrank - 1; r >= 0; r--) {
    for(int i = numr - 1; i >= 0; i--) {
      gaussian(x,y);
      YL[ r + i*rrank ] = (factor_type == 1) ? sign(x) : x;
      YL[ r + i*rrank ] /= matrix_scale;
    }
    // Add in R
    for(int i = numc - 1; i >= 0; i--) {
      gaussian(x,y);
      YR[ r + i*rrank ] = (factor_type == 1) ? sign(x) : x;
      YR[ r + i*rrank ] /= matrix_scale;
    }    
  }
  //std::cerr << "Finished constructing L and R" << std::endl;

  // Sample the entries
  int nEntries = numr*numc;
  // NB: THIS IS REPLCIATED IN THE EXPERIMENT SCRIPTS!
  int nSamples = (int) (oversample*rrank*(numr + numc - rrank));
  int nProbes  = int(ceil(nSamples*0.01));
  
  double noise_std = sqrt(noise_var);

  // create the train file
  sample_to_file(train_fname, nSamples, numr, numc, rrank, YL, YR, noise_std);

  // create the probe file
  sample_to_file(probe_fname, nProbes, numr, numc, rrank, YL, YR, 0.0);
  return nEntries;
}
