/* Authors: Christopher Re and Ben Recht
Copyright 2010-2011 Christopher Re and Ben Recht

This code is released under the GPLv3 License.
http://www.gnu.org/licenses/gpl.html

The files in this directory and its subdirectories are part of Jellyfish.

Jellyifsh is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Jellyfish is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jellyfish.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <cstdlib>
#include <iostream>
#include <cmath>

#include "fvector.h"

int FVector::_default_n = 30;

void FVector::init(int n) {
  _n = n;
  _p = new double[n];
  for(int i = 0; i < _n; i++) {
    _p[i] = (drand48() - 0.5) * 1e-3;
  }
}


FVector::FVector()  { init(_default_n); }
FVector::FVector(int n) { init(n); }

void FVector::zero() {
  for(int i = 0; i < _n; i++) {
    _p[i] = 0.0;
  }
}

FVector::~FVector() {
  delete [] _p;
}
  

FVector::FVector(const FVector &x) {
  _n = x._n;
  _p = new double[_n];
  for(int i = 0; i < _n; i++) {
    _p[i] = x._p[i];
  }
}

FVector&
FVector::operator=(const FVector &x) {
  if(_n != x._n) {
    // This should never happen...
    if(_p) { delete [] _p; }
    _p = new double[_n];
  }
   
  // simply overwrite.
  for(int i = 0; i < _n; i++) {
    _p[i] = x._p[i];
  } 
  return *this;
}


  
  

void 
FVector::
set(int i, double v) { if(i >= 0 && i < _n) { _p[i] = v;} else { std::cout << "[set] asked for " << i << std::endl; exit(-1); } }

double 
FVector::
get(int i) { if(i >= 0 && i < _n)  {return _p[i];} else { std::cout << "[get] asked for " << i << std::endl; exit(-1); } }

// void 
// FVector::scale(double w) { for(int i = 0; i < _n; i++) { _p[i] *= w; } }

void 
FVector::
subtract(const FVector &them) {
  for(int i = 0; i < _n; i ++) {
    _p[i] -= them._p[i];
  }
}


// void 
// FVector::scale_and_add(const FVector &them, double w) {
//   for(int i = 0; i < _n; i ++) {
//     _p[i] += them._p[i] * w;
//   }
// }

void 
FVector::ball_project(double B) {
  ball_project(B, B*B);
}

void 
FVector::ball_project(double B, double B_squared) {
  double norm_square = 0.0;
  for(int i = 0; i < _n; i++) {
    norm_square += _p[i]*_p[i];
  }
  if (norm_square > B_squared) {
    double norm = sqrt(norm_square);
    for(int i = 0; i < _n; i++) {
      _p[i] *= B/norm;
    }
  }
}


    
  
// double 
// FVector::dot(const FVector &v, const FVector &v2) {
//   if(v._n != v2._n) {
//     std::cout << "dot of two unequal vectors! " << v._n << " v. " << v2._n << std::endl;
//     exit(-1);
//   }
//   double ret = 0.0;
//   for(int i = 0; i < v._n; i++) {
//     ret += v._p[i] * v2._p[i];
//   }
//   return ret;
// }

int 
FVector::
get_size() { return _n; } 

double
FVector::
norm() { return sqrt(dot(*this,*this)); }

FVector* 
allocate_fvector_array(int ns) { return new FVector[ns]; }
