/* Authors: Christopher Re and Ben Recht
Copyright 2010-2011 Christopher Re and Ben Recht

This code is released under the GPLv3 License.
http://www.gnu.org/licenses/gpl.html

The files in this directory and its subdirectories are part of Jellyfish.

Jellyifsh is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Jellyfish is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jellyfish.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _fvector_h_
#define _fvector_h_ 1
#include <iostream>
#include <cstdlib>
class FVector {
  double* _p; 
  int _n;
  void init(int n);
public:
  static int _default_n;
  // This is really disgusting, but the default constructor intializes (initialization to 30)
  // This is is a hack to make stacking easy.
  FVector();  
  FVector(int n); 

  FVector(const FVector &x);
  FVector& operator=(const FVector &x);
  ~FVector();

  void zero(); 
  void set(int i, double v);
  double get(int i);
  inline void scale(double w) { for(register int i = _n - 1; i >=0; i--) { _p[i] *= w; } }
  void subtract(const FVector &them);
  inline void scale_and_add(const FVector &them, double w) {
      for(register int i = 0; i < _n; i ++) {
        _p[i] += them._p[i] * w;
      }    
  }
  void ball_project(double B);
  void ball_project(double B, double B_squared);
  static inline double dot(const FVector &v, const FVector &v2) {
    if(v._n != v2._n) {
      std::cout << "dot of two unequal vectors! " << v._n << " v. " << v2._n << std::endl;
      exit(-1);
    }
    double ret = 0.0;
    for(register int i = 0; i < v._n; i++) {
      ret += v._p[i] * v2._p[i];
    }
    return ret;
  }

  int get_size();
  double norm();
  // Helpers for NumPy code
  void set_size(int n) { FVector::_default_n = n;} 
};
FVector* allocate_fvector_array(int ns);
#endif
